@@ -277,6 +277,10 @@ _restic_backup()
     flags+=("--host=")
     two_word_flags+=("-H")
     local_nonpersistent_flags+=("--host=")
+    flags+=("--iexclude=")
+    local_nonpersistent_flags+=("--iexclude=")
+    flags+=("--ignore-inode")
+    local_nonpersistent_flags+=("--ignore-inode")
     flags+=("--one-file-system")
     flags+=("-x")
     local_nonpersistent_flags+=("--one-file-system")
@@ -1222,6 +1226,10 @@ _restic_restore()
     flags+=("--host=")
     two_word_flags+=("-H")
     local_nonpersistent_flags+=("--host=")
+    flags+=("--iexclude=")
+    local_nonpersistent_flags+=("--iexclude=")
+    flags+=("--iinclude=")
+    local_nonpersistent_flags+=("--iinclude=")
     flags+=("--include=")
     two_word_flags+=("-i")
     local_nonpersistent_flags+=("--include=")
@@ -1324,6 +1332,9 @@ _restic_snapshots()
     flags+=("--compact")
     flags+=("-c")
     local_nonpersistent_flags+=("--compact")
+    flags+=("--group-by=")
+    two_word_flags+=("-g")
+    local_nonpersistent_flags+=("--group-by=")
     flags+=("--help")
     flags+=("-h")
     local_nonpersistent_flags+=("--help")
