@@ -17,21 +17,21 @@ func TestFilesWriterBasic(t *testing.T) {
 	f2 := dir + "/f2"
 
 	rtest.OK(t, w.writeToFile(f1, []byte{1}))
-	rtest.Equals(t, 1, w.writers.Len())
+	rtest.Equals(t, 1, len(w.cache))
 	rtest.Equals(t, 1, len(w.inprogress))
 
 	rtest.OK(t, w.writeToFile(f2, []byte{2}))
-	rtest.Equals(t, 1, w.writers.Len())
+	rtest.Equals(t, 1, len(w.cache))
 	rtest.Equals(t, 2, len(w.inprogress))
 
 	rtest.OK(t, w.writeToFile(f1, []byte{1}))
 	w.close(f1)
-	rtest.Equals(t, 0, w.writers.Len())
+	rtest.Equals(t, 0, len(w.cache))
 	rtest.Equals(t, 1, len(w.inprogress))
 
 	rtest.OK(t, w.writeToFile(f2, []byte{2}))
 	w.close(f2)
-	rtest.Equals(t, 0, w.writers.Len())
+	rtest.Equals(t, 0, len(w.cache))
 	rtest.Equals(t, 0, len(w.inprogress))
 
 	buf, err := ioutil.ReadFile(f1)
